# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.bill_data import BillData
from ..types.bill_options import BillOptions
from ..types.export_format import ExportFormat
from ..types.file import File
from ..types.file_content import FileContent
from ..types.force_customer_creation import ForceCustomerCreation
from ..types.idempotency_key import IdempotencyKey
from ..types.payor_data_request import PayorDataRequest
from .raw_client import AsyncRawInvoiceClient, RawInvoiceClient
from .types.get_invoice_record import GetInvoiceRecord
from .types.invoice_number_response import InvoiceNumberResponse
from .types.invoice_response_without_data import InvoiceResponseWithoutData
from .types.query_invoice_response import QueryInvoiceResponse
from .types.send_invoice_response import SendInvoiceResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class InvoiceClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawInvoiceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawInvoiceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawInvoiceClient
        """
        return self._raw_client

    def add_invoice(
        self,
        entry: str,
        *,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        invoice_data: typing.Optional[BillData] = OMIT,
        scheduled_options: typing.Optional[BillOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InvoiceResponseWithoutData:
        """
        Creates an invoice in an entrypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the customer/payor. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        invoice_data : typing.Optional[BillData]
            Object describing the invoice. Required for POST requests.

        scheduled_options : typing.Optional[BillOptions]
            Object with options for scheduled invoices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceResponseWithoutData
            Success

        Examples
        --------
        import datetime

        from payabli import BillData, BillItem, PayorDataRequest, payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.add_invoice(
            entry="8cfec329267",
            customer_data=PayorDataRequest(
                first_name="Tamara",
                last_name="Bagratoni",
                customer_number="3",
            ),
            invoice_data=BillData(
                items=[
                    BillItem(
                        item_product_name="Adventure Consult",
                        item_description="Consultation for Georgian tours",
                        item_cost=100.0,
                        item_qty=1,
                        item_mode=1,
                    ),
                    BillItem(
                        item_product_name="Deposit ",
                        item_description="Deposit for trip planning",
                        item_cost=882.37,
                        item_qty=1,
                    ),
                ],
                invoice_date=datetime.date.fromisoformat(
                    "2025-10-19",
                ),
                invoice_type=0,
                invoice_status=1,
                frequency="one-time",
                invoice_amount=982.37,
                discount=10.0,
                invoice_number="INV-3",
            ),
        )
        """
        _response = self._raw_client.add_invoice(
            entry,
            force_customer_creation=force_customer_creation,
            idempotency_key=idempotency_key,
            customer_data=customer_data,
            invoice_data=invoice_data,
            scheduled_options=scheduled_options,
            request_options=request_options,
        )
        return _response.data

    def delete_attached_from_invoice(
        self, filename: str, id_invoice: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceResponseWithoutData:
        """
        Deletes an invoice that's attached to a file.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a request to `/api/Invoice/{idInvoice}`. Here, the filename is `0_Bill.pdf``.
            "DocumentsRef": {
              "zipfile": "inva_269.zip",
              "filelist": [
                {
                  "originalName": "Bill.pdf",
                  "zipName": "0_Bill.pdf",
                  "descriptor": null
                }
              ]
            }

        id_invoice : int
            Invoice ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceResponseWithoutData
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.delete_attached_from_invoice(
            filename="0_Bill.pdf",
            id_invoice=23548884,
        )
        """
        _response = self._raw_client.delete_attached_from_invoice(filename, id_invoice, request_options=request_options)
        return _response.data

    def delete_invoice(
        self, id_invoice: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceResponseWithoutData:
        """
        Deletes a single invoice from an entrypoint.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceResponseWithoutData
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.delete_invoice(
            id_invoice=23548884,
        )
        """
        _response = self._raw_client.delete_invoice(id_invoice, request_options=request_options)
        return _response.data

    def edit_invoice(
        self,
        id_invoice: int,
        *,
        force_customer_creation: typing.Optional[bool] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        invoice_data: typing.Optional[BillData] = OMIT,
        scheduled_options: typing.Optional[BillOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InvoiceResponseWithoutData:
        """
        Updates details for a single invoice in an entrypoint.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        force_customer_creation : typing.Optional[bool]
            When `true`, the request creates a new customer record, regardless of whether customer identifiers match an existing customer.

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the customer/payor. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        invoice_data : typing.Optional[BillData]
            Object describing the invoice. Required for POST requests.

        scheduled_options : typing.Optional[BillOptions]
            Object with options for scheduled invoices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceResponseWithoutData
            Success

        Examples
        --------
        import datetime

        from payabli import BillData, BillItem, payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.edit_invoice(
            id_invoice=332,
            invoice_data=BillData(
                items=[
                    BillItem(
                        item_product_name="Deposit",
                        item_description="Deposit for trip planning",
                        item_cost=882.37,
                        item_qty=1,
                    )
                ],
                invoice_date=datetime.date.fromisoformat(
                    "2025-10-19",
                ),
                invoice_amount=982.37,
                invoice_number="INV-6",
            ),
        )
        """
        _response = self._raw_client.edit_invoice(
            id_invoice,
            force_customer_creation=force_customer_creation,
            customer_data=customer_data,
            invoice_data=invoice_data,
            scheduled_options=scheduled_options,
            request_options=request_options,
        )
        return _response.data

    def get_attached_file_from_invoice(
        self,
        filename: str,
        id_invoice: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FileContent:
        """
        Retrieves a file attached to an invoice.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in the response to a request to `/api/Invoice/{idInvoice}`. Here, the filename is `0_Bill.pdf``.
            ```
              "DocumentsRef": {
                "zipfile": "inva_269.zip",
                "filelist": [
                  {
                    "originalName": "Bill.pdf",
                    "zipName": "0_Bill.pdf",
                    "descriptor": null
                  }
                ]
              }
              ```

        id_invoice : int
            Invoice ID

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FileContent
            A successful response returns a binary file when `returnObject` is `false`. When `returnObject` is `true`, the response contains the file content as a Base64-encoded string in an object. Due to technical limitations, only the object response is documented here.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.get_attached_file_from_invoice(
            filename="filename",
            id_invoice=1,
        )
        """
        _response = self._raw_client.get_attached_file_from_invoice(
            filename, id_invoice, return_object=return_object, request_options=request_options
        )
        return _response.data

    def get_invoice(
        self, id_invoice: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInvoiceRecord:
        """
        Retrieves a single invoice by ID.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInvoiceRecord
            Success. Fields marked optional may return `null` if not set.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.get_invoice(
            id_invoice=23548884,
        )
        """
        _response = self._raw_client.get_invoice(id_invoice, request_options=request_options)
        return _response.data

    def get_invoice_number(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceNumberResponse:
        """
        Retrieves the next available invoice number for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceNumberResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.get_invoice_number(
            entry="8cfec329267",
        )
        """
        _response = self._raw_client.get_invoice_number(entry, request_options=request_options)
        return _response.data

    def list_invoices(
        self,
        entry: str,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryInvoiceResponse:
        """
        Returns a list of invoices for an entrypoint. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query

            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:

            - `invoiceDate` (gt, ge, lt, le, eq, ne)
            - `dueDate` (gt, ge, lt, le, eq, ne)
            - `sentDate` (gt, ge, lt, le, eq, ne)
            - `frequency` (in, nin,ne, eq)
            - `invoiceType` (eq, ne)
            - `payTerms` (in, nin, eq, ne)
            - `paypointId` (ne, eq)
            - `totalAmount` (gt, ge, lt, le, eq, ne)
            - `paidAmount` (gt, ge, lt, le, eq, ne)
            - `status` (in, nin, eq, ne)
            - `invoiceNumber` (ct, nct, eq, ne)
            - `purchaseOrder` (ct, nct, eq, ne)
            - `itemProductCode` (ct, nct)
            - `itemDescription` (ct, nct)
            - `customerFirstname` (ct, nct, eq, ne)
            - `customerLastname` (ct, nct, eq, ne)
            - `customerName` (ct, nct)
            - `customerId` (eq, ne)
            - `customerNumber` (ct, nct, eq, ne)
            - `customerCompanyname` (ct, nct, eq, ne)
            - `customerAddress` (ct, nct, eq, ne)
            - `customerCity` (ct, nct, eq, ne)
            - `customerZip` (ct, nct, eq, ne)
            - `customerState` (ct, nct, eq, ne)
            - `customerCountry` (ct, nct, eq, ne)
            - `customerPhone` (ct, nct, eq, ne)
            - `customerEmail` (ct, nct, eq, ne)
            - `customerShippingAddress` (ct, nct, eq, ne)
            - `customerShippingCity` (ct, nct, eq, ne)
            - `customerShippingZip` (ct, nct, eq, ne)
            - `customerShippingState` (ct, nct, eq, ne)
            - `customerShippingCountry` (ct, nct, eq, ne)
            - `orgId` (eq)
            - `paylinkId` (ne, eq)
            - `paypointLegal` (ne, eq, ct, nct)
            - `paypointDba` (ne, eq, ct, nct)
            - `orgName` (ne, eq, ct, nct)
            - `additional-xxx` (ne, eq, ct, nct) where xxx is the additional field name

            List of comparison accepted - enclosed between parentheses:

            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: totalAmount(gt)=20 return all records with totalAmount greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryInvoiceResponse
            Success. Fields marked optional may return `null` if not set.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.list_invoices(
            entry="8cfec329267",
            from_record=251,
            limit_record=0,
            sort_by="desc(field_name)",
        )
        """
        _response = self._raw_client.list_invoices(
            entry,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    def list_invoices_org(
        self,
        org_id: int,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryInvoiceResponse:
        """
        Returns a list of invoices for an org. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query

            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:

            - `invoiceDate` (gt, ge, lt, le, eq, ne)
            - `dueDate` (gt, ge, lt, le, eq, ne)
            - `sentDate` (gt, ge, lt, le, eq, ne)
            - `frequency` (in, nin,ne, eq)
            - `invoiceType` (eq, ne)
            - `payTerms` (in, nin, eq, ne)
            - `paypointId` (ne, eq)
            - `totalAmount` (gt, ge, lt, le, eq, ne)
            - `paidAmount` (gt, ge, lt, le, eq, ne)
            - `status` (in, nin, eq, ne)
            - `invoiceNumber` (ct, nct, eq, ne)
            - `purchaseOrder` (ct, nct, eq, ne)
            - `itemProductCode` (ct, nct)
            - `itemDescription` (ct, nct)
            - `customerFirstname` (ct, nct, eq, ne)
            - `customerLastname` (ct, nct, eq, ne)
            - `customerName` (ct, nct)
            - `customerId` (eq, ne)
            - `customerNumber` (ct, nct, eq, ne)
            - `customerCompanyname` (ct, nct, eq, ne)
            - `customerAddress` (ct, nct, eq, ne)
            - `customerCity` (ct, nct, eq, ne)
            - `customerZip` (ct, nct, eq, ne)
            - `customerState` (ct, nct, eq, ne)
            - `customerCountry` (ct, nct, eq, ne)
            - `customerPhone` (ct, nct, eq, ne)
            - `customerEmail` (ct, nct, eq, ne)
            - `customerShippingAddress` (ct, nct, eq, ne)
            - `customerShippingCity` (ct, nct, eq, ne)
            - `customerShippingZip` (ct, nct, eq, ne)
            - `customerShippingState` (ct, nct, eq, ne)
            - `customerShippingCountry` (ct, nct, eq, ne)
            - `orgId` (eq)
            - `paylinkId` (ne, eq)
            - `paypointLegal` (ne, eq, ct, nct)
            - `paypointDba` (ne, eq, ct, nct)
            - `orgName` (ne, eq, ct, nct)
            - `additional-xxx` (ne, eq, ct, nct) where xxx is the additional field name

            List of comparison accepted - enclosed between parentheses:

            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: totalAmount(gt)=20 return all records with totalAmount greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryInvoiceResponse
            Success. Fields marked optional may return `null` if not set.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.list_invoices_org(
            org_id=123,
            from_record=251,
            limit_record=0,
            sort_by="desc(field_name)",
        )
        """
        _response = self._raw_client.list_invoices_org(
            org_id,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    def send_invoice(
        self,
        id_invoice: int,
        *,
        attachfile: typing.Optional[bool] = None,
        mail_2: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SendInvoiceResponse:
        """
        Sends an invoice from an entrypoint via email.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        attachfile : typing.Optional[bool]
            When `true`, attaches a PDF version of invoice to the email.

        mail_2 : typing.Optional[str]
            Email address where the invoice will be sent to. If this parameter isn't included, Payabli uses the email address on file for the customer owner of the invoice.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SendInvoiceResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.send_invoice(
            id_invoice=23548884,
            attachfile=True,
            mail_2="tamara@example.com",
        )
        """
        _response = self._raw_client.send_invoice(
            id_invoice, attachfile=attachfile, mail_2=mail_2, request_options=request_options
        )
        return _response.data

    def get_invoice_pdf(self, id_invoice: int, *, request_options: typing.Optional[RequestOptions] = None) -> File:
        """
        Export a single invoice in PDF format.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        File
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.invoice.get_invoice_pdf(
            id_invoice=23548884,
        )
        """
        _response = self._raw_client.get_invoice_pdf(id_invoice, request_options=request_options)
        return _response.data


class AsyncInvoiceClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawInvoiceClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawInvoiceClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawInvoiceClient
        """
        return self._raw_client

    async def add_invoice(
        self,
        entry: str,
        *,
        force_customer_creation: typing.Optional[ForceCustomerCreation] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        invoice_data: typing.Optional[BillData] = OMIT,
        scheduled_options: typing.Optional[BillOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InvoiceResponseWithoutData:
        """
        Creates an invoice in an entrypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        force_customer_creation : typing.Optional[ForceCustomerCreation]

        idempotency_key : typing.Optional[IdempotencyKey]

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the customer/payor. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        invoice_data : typing.Optional[BillData]
            Object describing the invoice. Required for POST requests.

        scheduled_options : typing.Optional[BillOptions]
            Object with options for scheduled invoices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceResponseWithoutData
            Success

        Examples
        --------
        import asyncio
        import datetime

        from payabli import Asyncpayabli, BillData, BillItem, PayorDataRequest

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.add_invoice(
                entry="8cfec329267",
                customer_data=PayorDataRequest(
                    first_name="Tamara",
                    last_name="Bagratoni",
                    customer_number="3",
                ),
                invoice_data=BillData(
                    items=[
                        BillItem(
                            item_product_name="Adventure Consult",
                            item_description="Consultation for Georgian tours",
                            item_cost=100.0,
                            item_qty=1,
                            item_mode=1,
                        ),
                        BillItem(
                            item_product_name="Deposit ",
                            item_description="Deposit for trip planning",
                            item_cost=882.37,
                            item_qty=1,
                        ),
                    ],
                    invoice_date=datetime.date.fromisoformat(
                        "2025-10-19",
                    ),
                    invoice_type=0,
                    invoice_status=1,
                    frequency="one-time",
                    invoice_amount=982.37,
                    discount=10.0,
                    invoice_number="INV-3",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_invoice(
            entry,
            force_customer_creation=force_customer_creation,
            idempotency_key=idempotency_key,
            customer_data=customer_data,
            invoice_data=invoice_data,
            scheduled_options=scheduled_options,
            request_options=request_options,
        )
        return _response.data

    async def delete_attached_from_invoice(
        self, filename: str, id_invoice: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceResponseWithoutData:
        """
        Deletes an invoice that's attached to a file.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in response to a request to `/api/Invoice/{idInvoice}`. Here, the filename is `0_Bill.pdf``.
            "DocumentsRef": {
              "zipfile": "inva_269.zip",
              "filelist": [
                {
                  "originalName": "Bill.pdf",
                  "zipName": "0_Bill.pdf",
                  "descriptor": null
                }
              ]
            }

        id_invoice : int
            Invoice ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceResponseWithoutData
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.delete_attached_from_invoice(
                filename="0_Bill.pdf",
                id_invoice=23548884,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_attached_from_invoice(
            filename, id_invoice, request_options=request_options
        )
        return _response.data

    async def delete_invoice(
        self, id_invoice: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceResponseWithoutData:
        """
        Deletes a single invoice from an entrypoint.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceResponseWithoutData
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.delete_invoice(
                id_invoice=23548884,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_invoice(id_invoice, request_options=request_options)
        return _response.data

    async def edit_invoice(
        self,
        id_invoice: int,
        *,
        force_customer_creation: typing.Optional[bool] = None,
        customer_data: typing.Optional[PayorDataRequest] = OMIT,
        invoice_data: typing.Optional[BillData] = OMIT,
        scheduled_options: typing.Optional[BillOptions] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> InvoiceResponseWithoutData:
        """
        Updates details for a single invoice in an entrypoint.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        force_customer_creation : typing.Optional[bool]
            When `true`, the request creates a new customer record, regardless of whether customer identifiers match an existing customer.

        customer_data : typing.Optional[PayorDataRequest]
            Object describing the customer/payor. Required for POST requests. Which fields are required depends on the paypoint's custom identifier settings.

        invoice_data : typing.Optional[BillData]
            Object describing the invoice. Required for POST requests.

        scheduled_options : typing.Optional[BillOptions]
            Object with options for scheduled invoices.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceResponseWithoutData
            Success

        Examples
        --------
        import asyncio
        import datetime

        from payabli import Asyncpayabli, BillData, BillItem

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.edit_invoice(
                id_invoice=332,
                invoice_data=BillData(
                    items=[
                        BillItem(
                            item_product_name="Deposit",
                            item_description="Deposit for trip planning",
                            item_cost=882.37,
                            item_qty=1,
                        )
                    ],
                    invoice_date=datetime.date.fromisoformat(
                        "2025-10-19",
                    ),
                    invoice_amount=982.37,
                    invoice_number="INV-6",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.edit_invoice(
            id_invoice,
            force_customer_creation=force_customer_creation,
            customer_data=customer_data,
            invoice_data=invoice_data,
            scheduled_options=scheduled_options,
            request_options=request_options,
        )
        return _response.data

    async def get_attached_file_from_invoice(
        self,
        filename: str,
        id_invoice: int,
        *,
        return_object: typing.Optional[bool] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> FileContent:
        """
        Retrieves a file attached to an invoice.

        Parameters
        ----------
        filename : str
            The filename in Payabli. Filename is `zipName` in the response to a request to `/api/Invoice/{idInvoice}`. Here, the filename is `0_Bill.pdf``.
            ```
              "DocumentsRef": {
                "zipfile": "inva_269.zip",
                "filelist": [
                  {
                    "originalName": "Bill.pdf",
                    "zipName": "0_Bill.pdf",
                    "descriptor": null
                  }
                ]
              }
              ```

        id_invoice : int
            Invoice ID

        return_object : typing.Optional[bool]
            When `true`, the request returns the file content as a Base64-encoded string.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        FileContent
            A successful response returns a binary file when `returnObject` is `false`. When `returnObject` is `true`, the response contains the file content as a Base64-encoded string in an object. Due to technical limitations, only the object response is documented here.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.get_attached_file_from_invoice(
                filename="filename",
                id_invoice=1,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_attached_file_from_invoice(
            filename, id_invoice, return_object=return_object, request_options=request_options
        )
        return _response.data

    async def get_invoice(
        self, id_invoice: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> GetInvoiceRecord:
        """
        Retrieves a single invoice by ID.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        GetInvoiceRecord
            Success. Fields marked optional may return `null` if not set.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.get_invoice(
                id_invoice=23548884,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_invoice(id_invoice, request_options=request_options)
        return _response.data

    async def get_invoice_number(
        self, entry: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> InvoiceNumberResponse:
        """
        Retrieves the next available invoice number for a paypoint.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        InvoiceNumberResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.get_invoice_number(
                entry="8cfec329267",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_invoice_number(entry, request_options=request_options)
        return _response.data

    async def list_invoices(
        self,
        entry: str,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryInvoiceResponse:
        """
        Returns a list of invoices for an entrypoint. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        entry : str
            The paypoint's entrypoint identifier. [Learn more](/api-reference/api-overview#entrypoint-vs-entry)

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query

            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:

            - `invoiceDate` (gt, ge, lt, le, eq, ne)
            - `dueDate` (gt, ge, lt, le, eq, ne)
            - `sentDate` (gt, ge, lt, le, eq, ne)
            - `frequency` (in, nin,ne, eq)
            - `invoiceType` (eq, ne)
            - `payTerms` (in, nin, eq, ne)
            - `paypointId` (ne, eq)
            - `totalAmount` (gt, ge, lt, le, eq, ne)
            - `paidAmount` (gt, ge, lt, le, eq, ne)
            - `status` (in, nin, eq, ne)
            - `invoiceNumber` (ct, nct, eq, ne)
            - `purchaseOrder` (ct, nct, eq, ne)
            - `itemProductCode` (ct, nct)
            - `itemDescription` (ct, nct)
            - `customerFirstname` (ct, nct, eq, ne)
            - `customerLastname` (ct, nct, eq, ne)
            - `customerName` (ct, nct)
            - `customerId` (eq, ne)
            - `customerNumber` (ct, nct, eq, ne)
            - `customerCompanyname` (ct, nct, eq, ne)
            - `customerAddress` (ct, nct, eq, ne)
            - `customerCity` (ct, nct, eq, ne)
            - `customerZip` (ct, nct, eq, ne)
            - `customerState` (ct, nct, eq, ne)
            - `customerCountry` (ct, nct, eq, ne)
            - `customerPhone` (ct, nct, eq, ne)
            - `customerEmail` (ct, nct, eq, ne)
            - `customerShippingAddress` (ct, nct, eq, ne)
            - `customerShippingCity` (ct, nct, eq, ne)
            - `customerShippingZip` (ct, nct, eq, ne)
            - `customerShippingState` (ct, nct, eq, ne)
            - `customerShippingCountry` (ct, nct, eq, ne)
            - `orgId` (eq)
            - `paylinkId` (ne, eq)
            - `paypointLegal` (ne, eq, ct, nct)
            - `paypointDba` (ne, eq, ct, nct)
            - `orgName` (ne, eq, ct, nct)
            - `additional-xxx` (ne, eq, ct, nct) where xxx is the additional field name

            List of comparison accepted - enclosed between parentheses:

            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: totalAmount(gt)=20 return all records with totalAmount greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryInvoiceResponse
            Success. Fields marked optional may return `null` if not set.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.list_invoices(
                entry="8cfec329267",
                from_record=251,
                limit_record=0,
                sort_by="desc(field_name)",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_invoices(
            entry,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    async def list_invoices_org(
        self,
        org_id: int,
        *,
        export_format: typing.Optional[ExportFormat] = None,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> QueryInvoiceResponse:
        """
        Returns a list of invoices for an org. Use filters to limit results. Include the `exportFormat` query parameter to return the results as a file instead of a JSON response.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        export_format : typing.Optional[ExportFormat]

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]
            Collection of field names, conditions, and values used to filter the query

            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:

            - `invoiceDate` (gt, ge, lt, le, eq, ne)
            - `dueDate` (gt, ge, lt, le, eq, ne)
            - `sentDate` (gt, ge, lt, le, eq, ne)
            - `frequency` (in, nin,ne, eq)
            - `invoiceType` (eq, ne)
            - `payTerms` (in, nin, eq, ne)
            - `paypointId` (ne, eq)
            - `totalAmount` (gt, ge, lt, le, eq, ne)
            - `paidAmount` (gt, ge, lt, le, eq, ne)
            - `status` (in, nin, eq, ne)
            - `invoiceNumber` (ct, nct, eq, ne)
            - `purchaseOrder` (ct, nct, eq, ne)
            - `itemProductCode` (ct, nct)
            - `itemDescription` (ct, nct)
            - `customerFirstname` (ct, nct, eq, ne)
            - `customerLastname` (ct, nct, eq, ne)
            - `customerName` (ct, nct)
            - `customerId` (eq, ne)
            - `customerNumber` (ct, nct, eq, ne)
            - `customerCompanyname` (ct, nct, eq, ne)
            - `customerAddress` (ct, nct, eq, ne)
            - `customerCity` (ct, nct, eq, ne)
            - `customerZip` (ct, nct, eq, ne)
            - `customerState` (ct, nct, eq, ne)
            - `customerCountry` (ct, nct, eq, ne)
            - `customerPhone` (ct, nct, eq, ne)
            - `customerEmail` (ct, nct, eq, ne)
            - `customerShippingAddress` (ct, nct, eq, ne)
            - `customerShippingCity` (ct, nct, eq, ne)
            - `customerShippingZip` (ct, nct, eq, ne)
            - `customerShippingState` (ct, nct, eq, ne)
            - `customerShippingCountry` (ct, nct, eq, ne)
            - `orgId` (eq)
            - `paylinkId` (ne, eq)
            - `paypointLegal` (ne, eq, ct, nct)
            - `paypointDba` (ne, eq, ct, nct)
            - `orgName` (ne, eq, ct, nct)
            - `additional-xxx` (ne, eq, ct, nct) where xxx is the additional field name

            List of comparison accepted - enclosed between parentheses:

            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: totalAmount(gt)=20 return all records with totalAmount greater than 20.00

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        QueryInvoiceResponse
            Success. Fields marked optional may return `null` if not set.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.list_invoices_org(
                org_id=123,
                from_record=251,
                limit_record=0,
                sort_by="desc(field_name)",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_invoices_org(
            org_id,
            export_format=export_format,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    async def send_invoice(
        self,
        id_invoice: int,
        *,
        attachfile: typing.Optional[bool] = None,
        mail_2: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> SendInvoiceResponse:
        """
        Sends an invoice from an entrypoint via email.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        attachfile : typing.Optional[bool]
            When `true`, attaches a PDF version of invoice to the email.

        mail_2 : typing.Optional[str]
            Email address where the invoice will be sent to. If this parameter isn't included, Payabli uses the email address on file for the customer owner of the invoice.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        SendInvoiceResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.send_invoice(
                id_invoice=23548884,
                attachfile=True,
                mail_2="tamara@example.com",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.send_invoice(
            id_invoice, attachfile=attachfile, mail_2=mail_2, request_options=request_options
        )
        return _response.data

    async def get_invoice_pdf(
        self, id_invoice: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> File:
        """
        Export a single invoice in PDF format.

        Parameters
        ----------
        id_invoice : int
            Invoice ID

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        File
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.invoice.get_invoice_pdf(
                id_invoice=23548884,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_invoice_pdf(id_invoice, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.boarding_link_api_response import BoardingLinkApiResponse
from ..types.orgid import Orgid
from ..types.payabli_api_response_template_id import PayabliApiResponseTemplateId
from ..types.template_code import TemplateCode
from ..types.template_content import TemplateContent
from ..types.template_name import TemplateName
from ..types.template_query_record import TemplateQueryRecord
from ..types.template_query_response import TemplateQueryResponse
from .raw_client import AsyncRawTemplatesClient, RawTemplatesClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class TemplatesClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawTemplatesClient
        """
        return self._raw_client

    def add_template(
        self,
        org_id_: int,
        *,
        org_id: typing.Optional[Orgid] = OMIT,
        pricing_id: typing.Optional[int] = OMIT,
        template_code: typing.Optional[TemplateCode] = OMIT,
        template_content: typing.Optional[TemplateContent] = OMIT,
        template_description: typing.Optional[str] = OMIT,
        template_name: typing.Optional[TemplateName] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseTemplateId:
        """
        Creates a boarding template in an organization.

        Parameters
        ----------
        org_id_ : int
            The numeric identifier for organization, assigned by Payabli.

        org_id : typing.Optional[Orgid]
            The ID of the organization the template belongs to.

        pricing_id : typing.Optional[int]

        template_code : typing.Optional[TemplateCode]

        template_content : typing.Optional[TemplateContent]

        template_description : typing.Optional[str]
            A description for the template.

        template_name : typing.Optional[TemplateName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseTemplateId
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.templates.add_template(
            org_id_=123,
        )
        """
        _response = self._raw_client.add_template(
            org_id_,
            org_id=org_id,
            pricing_id=pricing_id,
            template_code=template_code,
            template_content=template_content,
            template_description=template_description,
            template_name=template_name,
            request_options=request_options,
        )
        return _response.data

    def delete_template(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseTemplateId:
        """
        Deletes a template by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseTemplateId
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.templates.delete_template(
            template_id=80.0,
        )
        """
        _response = self._raw_client.delete_template(template_id, request_options=request_options)
        return _response.data

    def getlink_template(
        self, ignore_empty: bool, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BoardingLinkApiResponse:
        """
        Generates a boarding link from a boarding template.

        Parameters
        ----------
        ignore_empty : bool
            Ignore read-only and empty fields Default is `false`. If `ignoreEmpty` = `false` and any field is empty, then the request returns a failure response. If `ignoreEmpty` = `true`, the request returns the boarding link name regardless of whether fields are empty.

        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BoardingLinkApiResponse
            Success.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.templates.getlink_template(
            ignore_empty=True,
            template_id=80.0,
        )
        """
        _response = self._raw_client.getlink_template(ignore_empty, template_id, request_options=request_options)
        return _response.data

    def get_template(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TemplateQueryRecord:
        """
        Retrieves a boarding template's details by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplateQueryRecord
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.templates.get_template(
            template_id=80.0,
        )
        """
        _response = self._raw_client.get_template(template_id, request_options=request_options)
        return _response.data

    def list_templates(
        self,
        org_id: int,
        *,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplateQueryResponse:
        """
        Retrieves a list of boarding templates for an organization. Use filters to limit results. You can't make a request that includes filters from the API console in the documentation. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]

            Collection of field names, conditions, and values used to filter the query.

            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>


            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `createdAt` (gt, ge, lt, le, eq, ne)
            - `title` (ct, nct)
            - `description` (ct, nct)
            - `code` (ct, nct)
            - `orgParentname` (ct, nct)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: title(ct)=hoa return all records with title containing "hoa"

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplateQueryResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.templates.list_templates(
            org_id=123,
            from_record=251,
            limit_record=0,
            sort_by="desc(field_name)",
        )
        """
        _response = self._raw_client.list_templates(
            org_id,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    def update_template(
        self,
        template_id: float,
        *,
        org_id: typing.Optional[Orgid] = OMIT,
        pricing_id: typing.Optional[int] = OMIT,
        template_code: typing.Optional[TemplateCode] = OMIT,
        template_content: typing.Optional[TemplateContent] = OMIT,
        template_description: typing.Optional[str] = OMIT,
        template_name: typing.Optional[TemplateName] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseTemplateId:
        """
        Updates a boarding template by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        org_id : typing.Optional[Orgid]
            The ID of the organization the template belongs to.

        pricing_id : typing.Optional[int]

        template_code : typing.Optional[TemplateCode]

        template_content : typing.Optional[TemplateContent]

        template_description : typing.Optional[str]
            A description for the template.

        template_name : typing.Optional[TemplateName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseTemplateId
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.templates.update_template(
            template_id=80.0,
        )
        """
        _response = self._raw_client.update_template(
            template_id,
            org_id=org_id,
            pricing_id=pricing_id,
            template_code=template_code,
            template_content=template_content,
            template_description=template_description,
            template_name=template_name,
            request_options=request_options,
        )
        return _response.data


class AsyncTemplatesClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawTemplatesClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawTemplatesClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawTemplatesClient
        """
        return self._raw_client

    async def add_template(
        self,
        org_id_: int,
        *,
        org_id: typing.Optional[Orgid] = OMIT,
        pricing_id: typing.Optional[int] = OMIT,
        template_code: typing.Optional[TemplateCode] = OMIT,
        template_content: typing.Optional[TemplateContent] = OMIT,
        template_description: typing.Optional[str] = OMIT,
        template_name: typing.Optional[TemplateName] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseTemplateId:
        """
        Creates a boarding template in an organization.

        Parameters
        ----------
        org_id_ : int
            The numeric identifier for organization, assigned by Payabli.

        org_id : typing.Optional[Orgid]
            The ID of the organization the template belongs to.

        pricing_id : typing.Optional[int]

        template_code : typing.Optional[TemplateCode]

        template_content : typing.Optional[TemplateContent]

        template_description : typing.Optional[str]
            A description for the template.

        template_name : typing.Optional[TemplateName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseTemplateId
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.templates.add_template(
                org_id_=123,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_template(
            org_id_,
            org_id=org_id,
            pricing_id=pricing_id,
            template_code=template_code,
            template_content=template_content,
            template_description=template_description,
            template_name=template_name,
            request_options=request_options,
        )
        return _response.data

    async def delete_template(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponseTemplateId:
        """
        Deletes a template by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseTemplateId
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.templates.delete_template(
                template_id=80.0,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.delete_template(template_id, request_options=request_options)
        return _response.data

    async def getlink_template(
        self, ignore_empty: bool, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BoardingLinkApiResponse:
        """
        Generates a boarding link from a boarding template.

        Parameters
        ----------
        ignore_empty : bool
            Ignore read-only and empty fields Default is `false`. If `ignoreEmpty` = `false` and any field is empty, then the request returns a failure response. If `ignoreEmpty` = `true`, the request returns the boarding link name regardless of whether fields are empty.

        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BoardingLinkApiResponse
            Success.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.templates.getlink_template(
                ignore_empty=True,
                template_id=80.0,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.getlink_template(ignore_empty, template_id, request_options=request_options)
        return _response.data

    async def get_template(
        self, template_id: float, *, request_options: typing.Optional[RequestOptions] = None
    ) -> TemplateQueryRecord:
        """
        Retrieves a boarding template's details by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplateQueryRecord
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.templates.get_template(
                template_id=80.0,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_template(template_id, request_options=request_options)
        return _response.data

    async def list_templates(
        self,
        org_id: int,
        *,
        from_record: typing.Optional[int] = None,
        limit_record: typing.Optional[int] = None,
        parameters: typing.Optional[typing.Dict[str, typing.Optional[str]]] = None,
        sort_by: typing.Optional[str] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> TemplateQueryResponse:
        """
        Retrieves a list of boarding templates for an organization. Use filters to limit results. You can't make a request that includes filters from the API console in the documentation. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

        Parameters
        ----------
        org_id : int
            The numeric identifier for organization, assigned by Payabli.

        from_record : typing.Optional[int]
            The number of records to skip before starting to collect the result set.

        limit_record : typing.Optional[int]
            Max number of records to return for the query. Use `0` or negative value to return all records.

        parameters : typing.Optional[typing.Dict[str, typing.Optional[str]]]

            Collection of field names, conditions, and values used to filter the query.

            <Info>
              **You must remove `parameters=` from the request before you send it, otherwise Payabli will ignore the filters.**

              Because of a technical limitation, you can't make a request that includes filters from the API console on this page. The response won't be filtered. Instead, copy the request, remove `parameters=` and run the request in a different client.

              For example:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?parameters=totalAmount(gt)=1000&limitRecord=20

              should become:

              --url https://api-sandbox.payabli.com/api/Query/transactions/org/236?totalAmount(gt)=1000&limitRecord=20
            </Info>


            See [Filters and Conditions Reference](/developers/developer-guides/pay-ops-reporting-engine-overview#filters-and-conditions-reference) for help.

            List of field names accepted:
            - `createdAt` (gt, ge, lt, le, eq, ne)
            - `title` (ct, nct)
            - `description` (ct, nct)
            - `code` (ct, nct)
            - `orgParentname` (ct, nct)

            List of comparison accepted - enclosed between parentheses:
            - eq or empty => equal
            - gt => greater than
            - ge => greater or equal
            - lt => less than
            - le => less or equal
            - ne => not equal
            - ct => contains
            - nct => not contains
            - in => inside array
            - nin => not inside array

            List of parameters accepted:
            - limitRecord : max number of records for query (default="20", "0" or negative value for all)
            - fromRecord : initial record in query

            Example: title(ct)=hoa return all records with title containing "hoa"

        sort_by : typing.Optional[str]
            The field name to use for sorting results. Use `desc(field_name)` to sort descending by `field_name`, and use `asc(field_name)` to sort ascending by `field_name`.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        TemplateQueryResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.templates.list_templates(
                org_id=123,
                from_record=251,
                limit_record=0,
                sort_by="desc(field_name)",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.list_templates(
            org_id,
            from_record=from_record,
            limit_record=limit_record,
            parameters=parameters,
            sort_by=sort_by,
            request_options=request_options,
        )
        return _response.data

    async def update_template(
        self,
        template_id: float,
        *,
        org_id: typing.Optional[Orgid] = OMIT,
        pricing_id: typing.Optional[int] = OMIT,
        template_code: typing.Optional[TemplateCode] = OMIT,
        template_content: typing.Optional[TemplateContent] = OMIT,
        template_description: typing.Optional[str] = OMIT,
        template_name: typing.Optional[TemplateName] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> PayabliApiResponseTemplateId:
        """
        Updates a boarding template by ID.

        Parameters
        ----------
        template_id : float
            The boarding template ID. Can be found at the end of the boarding template URL in PartnerHub. Example: `https://partner-sandbox.payabli.com/myorganization/boarding/edittemplate/80`. Here, the template ID is `80`.

        org_id : typing.Optional[Orgid]
            The ID of the organization the template belongs to.

        pricing_id : typing.Optional[int]

        template_code : typing.Optional[TemplateCode]

        template_content : typing.Optional[TemplateContent]

        template_description : typing.Optional[str]
            A description for the template.

        template_name : typing.Optional[TemplateName]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponseTemplateId
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.templates.update_template(
                template_id=80.0,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.update_template(
            template_id,
            org_id=org_id,
            pricing_id=pricing_id,
            template_code=template_code,
            template_content=template_content,
            template_description=template_description,
            template_name=template_name,
            request_options=request_options,
        )
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from .bill_query_record_2_bill_approvals_item import BillQueryRecord2BillApprovalsItem

BillApprovals = typing.Optional[typing.List[typing.Optional[BillQueryRecord2BillApprovalsItem]]]



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .enabled import Enabled
from .label_element import LabelElement
from .order import Order


class InvoiceElement(UniversalBaseModel):
    enabled: typing.Optional[Enabled] = None
    invoice_link: typing_extensions.Annotated[typing.Optional[LabelElement], FieldMetadata(alias="invoiceLink")] = (
        pydantic.Field(default=None)
    )
    """
    Link to invoice
    """

    order: typing.Optional[Order] = None
    view_invoice_details: typing_extensions.Annotated[
        typing.Optional[LabelElement], FieldMetadata(alias="viewInvoiceDetails")
    ] = pydantic.Field(default=None)
    """
    Link to view invoice details
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .batch_number import BatchNumber
from .entrypointfield import Entrypointfield
from .external_paypoint_id import ExternalPaypointId


class QueryBatchesOutResponseRecordsItem(UniversalBaseModel):
    ach_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="AchAmount")] = pydantic.Field(
        default=None
    )
    """
    
    """

    ach_records: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="AchRecords")] = pydantic.Field(
        default=None
    )
    """
    
    """

    ach_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="AchStatus")] = pydantic.Field(
        default=None
    )
    """
    
    """

    ach_status_text: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="AchStatusText")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    batch_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The amount of the batch.
    """

    batch_cancelled_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="BatchCancelledAmount")
    ] = pydantic.Field(default=None)
    """
    
    """

    batch_cancelled_records: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="BatchCancelledRecords")
    ] = pydantic.Field(default=None)
    """
    
    """

    batch_date: typing_extensions.Annotated[typing.Optional[dt.datetime], FieldMetadata(alias="BatchDate")] = (
        pydantic.Field(default=None)
    )
    """
    The batch date.
    """

    batch_number: typing_extensions.Annotated[typing.Optional[BatchNumber], FieldMetadata(alias="BatchNumber")] = None
    batch_paid_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="BatchPaidAmount")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    batch_paid_records: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="BatchPaidRecords")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    batch_processed_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="BatchProcessedAmount")
    ] = pydantic.Field(default=None)
    """
    
    """

    batch_processed_records: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="BatchProcessedRecords")
    ] = pydantic.Field(default=None)
    """
    
    """

    batch_processing_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="BatchProcessingAmount")
    ] = pydantic.Field(default=None)
    """
    
    """

    batch_processing_records: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="BatchProcessingRecords")
    ] = pydantic.Field(default=None)
    """
    
    """

    batch_records: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="BatchRecords")] = (
        pydantic.Field(default=None)
    )
    """
    The number of records in the batch.
    """

    batch_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="BatchStatus")] = (
        pydantic.Field(default=None)
    )
    """
    The batch status. See [Batch Status](/developers/references/money-out-statuses#batch-statuses) for more.
    """

    batch_status_text: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="BatchStatusText")] = (
        pydantic.Field(default=None)
    )
    """
    A text description of the batch status.
    """

    card_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="CardAmount")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    card_records: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="CardRecords")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    card_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="CardStatus")] = pydantic.Field(
        default=None
    )
    """
    
    """

    card_status_text: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="CardStatusText")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    check_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="CheckAmount")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    check_records: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="CheckRecords")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    check_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="CheckStatus")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    check_status_text: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="CheckStatusText")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    entry_name: typing_extensions.Annotated[typing.Optional[Entrypointfield], FieldMetadata(alias="EntryName")] = None
    external_paypoint_id: typing_extensions.Annotated[
        typing.Optional[ExternalPaypointId], FieldMetadata(alias="externalPaypointID")
    ] = None
    id_batch: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="IdBatch")] = pydantic.Field(
        default=None
    )
    """
    The batch ID.
    """

    parent_org_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="ParentOrgName")] = (
        pydantic.Field(default=None)
    )
    """
    The entrypoint's parent org.
    """

    paypoint_dba: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointDba")] = (
        pydantic.Field(default=None)
    )
    """
    Paypoint DBA name.
    """

    paypoint_id: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="PaypointId")] = pydantic.Field(
        default=None
    )
    """
    Paypoint ID.
    """

    paypoint_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="PaypointName")] = (
        pydantic.Field(default=None)
    )
    """
    Paypoint legal name.
    """

    vcard_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="VcardAmount")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    vcard_records: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="VcardRecords")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    vcard_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="VcardStatus")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    vcard_status_text: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="VcardStatusText")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    wire_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="WireAmount")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    wire_records: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="WireRecords")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    wire_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="WireStatus")] = pydantic.Field(
        default=None
    )
    """
    
    """

    wire_status_text: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="WireStatusText")] = (
        pydantic.Field(default=None)
    )
    """
    
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class BinData(UniversalBaseModel):
    """
    Object containing information related to the card. This object is `null`
    unless the payment method is card. If the payment method is Apple Pay, the
    binData will be related to the DPAN (device primary account number), not
    the card connected to Apple Pay.

    Examples
    --------
    from payabli import BinData

    BinData(
        bin_matched_length="6",
        bin_card_brand="Visa",
        bin_card_type="Credit",
        bin_card_category="PLATINUM",
        bin_card_issuer="Bank of Example",
        bin_card_issuer_country="United States",
        bin_card_issuer_country_code_a_2="US",
        bin_card_issuer_country_number="840",
        bin_card_is_regulated="false",
        bin_card_use_category="Consumer",
        bin_card_issuer_country_code_a_3="USA",
    )
    """

    bin_matched_length: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="binMatchedLength")] = (
        pydantic.Field(default=None)
    )
    """
    The number of characters from the beginning of the card number that
    were matched against a Bank Identification Number (BIN) or the Card
    Range table.
    """

    bin_card_brand: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="binCardBrand")] = (
        pydantic.Field(default=None)
    )
    """
    The card brand. For example, Visa, Mastercard, American Express,
    Discover.
    """

    bin_card_type: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="binCardType")] = (
        pydantic.Field(default=None)
    )
    """
    The type of card: Credit or Debit.
    """

    bin_card_category: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="binCardCategory")] = (
        pydantic.Field(default=None)
    )
    """
    The category of the card, which indicates the card product. For example: Standard, Gold, Platinum, etc. The binCardCategory for prepaid cards is marked `PREPAID`.
    """

    bin_card_issuer: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="binCardIssuer")] = (
        pydantic.Field(default=None)
    )
    """
    The name of the financial institution that issued the card.
    """

    bin_card_issuer_country: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="binCardIssuerCountry")
    ] = pydantic.Field(default=None)
    """
    The issuing financial institution's country name.
    """

    bin_card_issuer_country_code_a_2: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="binCardIssuerCountryCodeA2")
    ] = pydantic.Field(default=None)
    """
    The issuing financial institution's two-character ISO country code. See [this resource](https://www.iso.org/obp/ui/#search) for a list of codes.
    """

    bin_card_issuer_country_number: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="binCardIssuerCountryNumber")
    ] = pydantic.Field(default=None)
    """
    The issuing financial institution's ISO standard numeric country code. See [this resource](https://www.iso.org/obp/ui/#search) for a list of codes.
    """

    bin_card_is_regulated: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="binCardIsRegulated")
    ] = pydantic.Field(default=None)
    """
    Indicates whether the card is regulated.
    """

    bin_card_use_category: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="binCardUseCategory")
    ] = pydantic.Field(default=None)
    """
    The use category classification for the card.
    """

    bin_card_issuer_country_code_a_3: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="binCardIssuerCountryCodeA3")
    ] = pydantic.Field(default=None)
    """
    The issuing financial institution's three-character ISO country code.
    See [this resource](https://www.iso.org/obp/ui/#search) for a list of
    codes.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class FrequencyList(UniversalBaseModel):
    annually: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable or disable frequency
    """

    every_2_weeks: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="every2Weeks")] = (
        pydantic.Field(default=None)
    )
    """
    Enable or disable frequency
    """

    every_3_months: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="every3Months")] = (
        pydantic.Field(default=None)
    )
    """
    Enable or disable frequency
    """

    every_6_months: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="every6Months")] = (
        pydantic.Field(default=None)
    )
    """
    Enable or disable frequency
    """

    monthly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable or disable frequency
    """

    one_time: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="one-time")] = pydantic.Field(
        default=None
    )
    """
    Enable or disable frequency
    """

    weekly: typing.Optional[bool] = pydantic.Field(default=None)
    """
    Enable or disable frequency
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PairFiles(UniversalBaseModel):
    original_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="originalName")] = (
        pydantic.Field(default=None)
    )
    """
    Original filename
    """

    zip_name: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="zipName")] = pydantic.Field(
        default=None
    )
    """
    Filename assigned to zipped file. This is the name to use for reference in the API functions to get files in attachments.
    """

    descriptor: typing.Optional[str] = pydantic.Field(default=None)
    """
    Descriptor of the file.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ...core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ...core.serialization import FieldMetadata
from ...types.contacts import Contacts
from .ocr_vendor_additional_data import OcrVendorAdditionalData
from .ocr_vendor_billing_data import OcrVendorBillingData


class OcrVendor(UniversalBaseModel):
    vendor_number: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="vendorNumber")] = None
    name_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="name1")] = None
    name_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="name2")] = None
    ein: typing.Optional[str] = None
    phone: typing.Optional[str] = None
    email: typing.Optional[str] = None
    address_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="address1")] = None
    address_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="address2")] = None
    city: typing.Optional[str] = None
    state: typing.Optional[str] = None
    zip: typing.Optional[str] = None
    country: typing.Optional[str] = None
    mcc: typing.Optional[str] = None
    location_code: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="locationCode")] = None
    contacts: typing.Optional[typing.List[Contacts]] = None
    billing_data: typing_extensions.Annotated[
        typing.Optional[OcrVendorBillingData], FieldMetadata(alias="billingData")
    ] = None
    payment_method: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="paymentMethod")] = None
    vendor_status: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="vendorStatus")] = None
    remit_address_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="remitAddress1")] = None
    remit_address_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="remitAddress2")] = None
    remit_city: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="remitCity")] = None
    remit_state: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="remitState")] = None
    remit_zip: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="remitZip")] = None
    remit_country: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="remitCountry")] = None
    payee_name_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="payeeName1")] = None
    payee_name_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="payeeName2")] = None
    customer_vendor_account: typing_extensions.Annotated[
        typing.Optional[str], FieldMetadata(alias="customerVendorAccount")
    ] = None
    internal_reference_id: typing_extensions.Annotated[
        typing.Optional[int], FieldMetadata(alias="internalReferenceId")
    ] = None
    custom_field_1: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField1")] = None
    custom_field_2: typing_extensions.Annotated[typing.Optional[str], FieldMetadata(alias="customField2")] = None
    additional_data: typing_extensions.Annotated[
        typing.Optional[OcrVendorAdditionalData], FieldMetadata(alias="additionalData")
    ] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .page_identifier import PageIdentifier


class QueryResponseSettlementsSummary(UniversalBaseModel):
    held_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="heldAmount")] = (
        pydantic.Field(default=None)
    )
    """
    Funds being held for fraud or risk concerns.
    """

    pageidentifier: typing.Optional[PageIdentifier] = None
    page_size: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="pageSize")] = pydantic.Field(
        default=None
    )
    """
    Number of records per page.
    """

    refunds: typing.Optional[float] = pydantic.Field(default=None)
    """
    Total refunds deducted from the transfer.
    """

    service_fees: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="serviceFees")] = (
        pydantic.Field(default=None)
    )
    """
    Service fees are any pass-through fees charged to the customer at the time of payment. These aren't transferred to the merchant when the batch is transferred and funded.
    """

    total_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="totalAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The total sum of the settlements in the response.
    """

    total_net_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="totalNetAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The total sum of the settlements in the response.
    """

    total_pages: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="totalPages")] = pydantic.Field(
        default=None
    )
    """
    Number of pages in the response.
    """

    total_records: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="totalRecords")] = (
        pydantic.Field(default=None)
    )
    """
    Number of records in the response.
    """

    transfer_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="transferAmount")] = (
        pydantic.Field(default=None)
    )
    """
    The transfer amount is the net batch amount plus or minus any returns, refunds, billing and fees items, chargebacks, adjustments, and third party payments. This is the amount from the batch that's transferred to the merchant bank account.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..money_out_types.types.auth_capture_payout_response import AuthCapturePayoutResponse
from ..money_out_types.types.authorize_payment_method import AuthorizePaymentMethod
from ..money_out_types.types.capture_all_out_response import CaptureAllOutResponse
from ..money_out_types.types.operation_result import OperationResult
from ..money_out_types.types.request_out_authorize_invoice_data import RequestOutAuthorizeInvoiceData
from ..money_out_types.types.request_out_authorize_payment_details import RequestOutAuthorizePaymentDetails
from ..money_out_types.types.request_out_authorize_vendor_data import RequestOutAuthorizeVendorData
from ..money_out_types.types.v_card_get_response import VCardGetResponse
from ..types.accountid import Accountid
from ..types.bill_detail_response import BillDetailResponse
from ..types.entrypointfield import Entrypointfield
from ..types.idempotency_key import IdempotencyKey
from ..types.order_id import OrderId
from ..types.orderdescription import Orderdescription
from ..types.payabli_api_response_0000 import PayabliApiResponse0000
from ..types.source import Source
from ..types.subdomain import Subdomain
from ..types.subscriptionid import Subscriptionid
from .raw_client import AsyncRawMoneyOutClient, RawMoneyOutClient

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class MoneyOutClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawMoneyOutClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawMoneyOutClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawMoneyOutClient
        """
        return self._raw_client

    def authorize_out(
        self,
        *,
        entry_point: Entrypointfield,
        payment_method: AuthorizePaymentMethod,
        payment_details: RequestOutAuthorizePaymentDetails,
        vendor_data: RequestOutAuthorizeVendorData,
        invoice_data: typing.Sequence[RequestOutAuthorizeInvoiceData],
        allow_duplicated_bills: typing.Optional[bool] = None,
        do_not_create_bills: typing.Optional[bool] = None,
        force_vendor_creation: typing.Optional[bool] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        source: typing.Optional[Source] = OMIT,
        order_id: typing.Optional[OrderId] = OMIT,
        order_description: typing.Optional[Orderdescription] = OMIT,
        account_id: typing.Optional[Accountid] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        subscription_id: typing.Optional[Subscriptionid] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AuthCapturePayoutResponse:
        """
        Authorizes transaction for payout. Authorized transactions aren't flagged for settlement until captured. Use `referenceId` returned in the response to capture the transaction.

        Parameters
        ----------
        entry_point : Entrypointfield

        payment_method : AuthorizePaymentMethod

        payment_details : RequestOutAuthorizePaymentDetails
            Object containing payment details.

        vendor_data : RequestOutAuthorizeVendorData
            Object containing vendor data.

        invoice_data : typing.Sequence[RequestOutAuthorizeInvoiceData]
            Array of bills associated to the transaction

        allow_duplicated_bills : typing.Optional[bool]
            When `true`, the authorization bypasses the requirement for unique bills, identified by vendor invoice number. This allows you to make more than one payout authorization for a bill, like a split payment.

        do_not_create_bills : typing.Optional[bool]
            When `true`, Payabli won't automatically create a bill for this payout transaction.

        force_vendor_creation : typing.Optional[bool]
            When `true`, the request creates a new vendor record, regardless of whether the vendor already exists.

        idempotency_key : typing.Optional[IdempotencyKey]

        source : typing.Optional[Source]

        order_id : typing.Optional[OrderId]

        order_description : typing.Optional[Orderdescription]

        account_id : typing.Optional[Accountid]

        subdomain : typing.Optional[Subdomain]

        subscription_id : typing.Optional[Subscriptionid]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthCapturePayoutResponse
            Success

        Examples
        --------
        from payabli import payabli
        from payabli.money_out_types import (
            AuthorizePaymentMethod,
            RequestOutAuthorizeInvoiceData,
            RequestOutAuthorizePaymentDetails,
            RequestOutAuthorizeVendorData,
        )

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.money_out.authorize_out(
            entry_point="48acde49",
            invoice_data=[
                RequestOutAuthorizeInvoiceData(
                    bill_id=54323,
                )
            ],
            order_description="Window Painting",
            payment_details=RequestOutAuthorizePaymentDetails(
                total_amount=47.0,
            ),
            payment_method=AuthorizePaymentMethod(
                method="managed",
            ),
            vendor_data=RequestOutAuthorizeVendorData(
                vendor_number="7895433",
            ),
        )
        """
        _response = self._raw_client.authorize_out(
            entry_point=entry_point,
            payment_method=payment_method,
            payment_details=payment_details,
            vendor_data=vendor_data,
            invoice_data=invoice_data,
            allow_duplicated_bills=allow_duplicated_bills,
            do_not_create_bills=do_not_create_bills,
            force_vendor_creation=force_vendor_creation,
            idempotency_key=idempotency_key,
            source=source,
            order_id=order_id,
            order_description=order_description,
            account_id=account_id,
            subdomain=subdomain,
            subscription_id=subscription_id,
            request_options=request_options,
        )
        return _response.data

    def cancel_all_out(
        self, *, request: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> CaptureAllOutResponse:
        """
        Cancels an array of payout transactions.

        Parameters
        ----------
        request : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CaptureAllOutResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.money_out.cancel_all_out(
            request=["2-29", "2-28", "2-27"],
        )
        """
        _response = self._raw_client.cancel_all_out(request=request, request_options=request_options)
        return _response.data

    def cancel_out(
        self, reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse0000:
        """
        Cancel a payout transaction by ID.

        Parameters
        ----------
        reference_id : str
            The ID for the payout transaction.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse0000
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.money_out.cancel_out(
            reference_id="129-219",
        )
        """
        _response = self._raw_client.cancel_out(reference_id, request_options=request_options)
        return _response.data

    def capture_all_out(
        self,
        *,
        request: typing.Sequence[str],
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CaptureAllOutResponse:
        """
        Captures an array of authorized payout transactions for settlement.

        Parameters
        ----------
        request : typing.Sequence[str]

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CaptureAllOutResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.money_out.capture_all_out(
            request=["2-29", "2-28", "2-27"],
        )
        """
        _response = self._raw_client.capture_all_out(
            request=request, idempotency_key=idempotency_key, request_options=request_options
        )
        return _response.data

    def capture_out(
        self,
        reference_id: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AuthCapturePayoutResponse:
        """
        Captures a single authorized payout transaction by ID.

        Parameters
        ----------
        reference_id : str
            The ID for the payout transaction.

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthCapturePayoutResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.money_out.capture_out(
            reference_id="129-219",
        )
        """
        _response = self._raw_client.capture_out(
            reference_id, idempotency_key=idempotency_key, request_options=request_options
        )
        return _response.data

    def payout_details(
        self, trans_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BillDetailResponse:
        """
        Returns details for a processed money out transaction.

        Parameters
        ----------
        trans_id : str
            ReferenceId for the transaction (PaymentId).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillDetailResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.money_out.payout_details(
            trans_id="45-as456777hhhhhhhhhh77777777-324",
        )
        """
        _response = self._raw_client.payout_details(trans_id, request_options=request_options)
        return _response.data

    def v_card_get(
        self, card_token: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> VCardGetResponse:
        """
        Retrieves vCard details for a single card in an entrypoint.

        Parameters
        ----------
        card_token : str
            ID for a virtual card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VCardGetResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.money_out.v_card_get(
            card_token="20230403315245421165",
        )
        """
        _response = self._raw_client.v_card_get(card_token, request_options=request_options)
        return _response.data

    def send_v_card_link(
        self, *, trans_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> OperationResult:
        """
        Sends a virtual card link via email to the vendor associated with the `transId`.

        Parameters
        ----------
        trans_id : str
            The transaction ID of the virtual card payout. The ID is returned as `ReferenceId` in the response when you authorize a payout with POST /MoneyOut/authorize.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OperationResult

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.money_out.send_v_card_link(
            trans_id="01K33Z6YQZ6GD5QVKZ856MJBSC",
        )
        """
        _response = self._raw_client.send_v_card_link(trans_id=trans_id, request_options=request_options)
        return _response.data

    def get_check_image(self, asset_name: str, *, request_options: typing.Optional[RequestOptions] = None) -> str:
        """
        Retrieve the image of a check associated with a processed transaction.
        The check image is returned in the response body as a base64-encoded string.
        The check image is only available for payouts that have been processed.

        Parameters
        ----------
        asset_name : str
            Name of the check asset to retrieve. This is returned as `filename` in the `CheckData` object
            in the response when you make a GET request to `/MoneyOut/details/{transId}`.
            ```
                "CheckData": {
                  "ftype": "PDF",
                  "filename": "check133832686289732320_01JKBNZ5P32JPTZY8XXXX000000.pdf",
                  "furl": "",
                  "fContent": ""
              }
            ```

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            A base64-encoded string of the check image.

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.money_out.get_check_image(
            asset_name="check133832686289732320_01JKBNZ5P32JPTZY8XXXX000000.pdf",
        )
        """
        _response = self._raw_client.get_check_image(asset_name, request_options=request_options)
        return _response.data


class AsyncMoneyOutClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawMoneyOutClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawMoneyOutClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawMoneyOutClient
        """
        return self._raw_client

    async def authorize_out(
        self,
        *,
        entry_point: Entrypointfield,
        payment_method: AuthorizePaymentMethod,
        payment_details: RequestOutAuthorizePaymentDetails,
        vendor_data: RequestOutAuthorizeVendorData,
        invoice_data: typing.Sequence[RequestOutAuthorizeInvoiceData],
        allow_duplicated_bills: typing.Optional[bool] = None,
        do_not_create_bills: typing.Optional[bool] = None,
        force_vendor_creation: typing.Optional[bool] = None,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        source: typing.Optional[Source] = OMIT,
        order_id: typing.Optional[OrderId] = OMIT,
        order_description: typing.Optional[Orderdescription] = OMIT,
        account_id: typing.Optional[Accountid] = OMIT,
        subdomain: typing.Optional[Subdomain] = OMIT,
        subscription_id: typing.Optional[Subscriptionid] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AuthCapturePayoutResponse:
        """
        Authorizes transaction for payout. Authorized transactions aren't flagged for settlement until captured. Use `referenceId` returned in the response to capture the transaction.

        Parameters
        ----------
        entry_point : Entrypointfield

        payment_method : AuthorizePaymentMethod

        payment_details : RequestOutAuthorizePaymentDetails
            Object containing payment details.

        vendor_data : RequestOutAuthorizeVendorData
            Object containing vendor data.

        invoice_data : typing.Sequence[RequestOutAuthorizeInvoiceData]
            Array of bills associated to the transaction

        allow_duplicated_bills : typing.Optional[bool]
            When `true`, the authorization bypasses the requirement for unique bills, identified by vendor invoice number. This allows you to make more than one payout authorization for a bill, like a split payment.

        do_not_create_bills : typing.Optional[bool]
            When `true`, Payabli won't automatically create a bill for this payout transaction.

        force_vendor_creation : typing.Optional[bool]
            When `true`, the request creates a new vendor record, regardless of whether the vendor already exists.

        idempotency_key : typing.Optional[IdempotencyKey]

        source : typing.Optional[Source]

        order_id : typing.Optional[OrderId]

        order_description : typing.Optional[Orderdescription]

        account_id : typing.Optional[Accountid]

        subdomain : typing.Optional[Subdomain]

        subscription_id : typing.Optional[Subscriptionid]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthCapturePayoutResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli
        from payabli.money_out_types import (
            AuthorizePaymentMethod,
            RequestOutAuthorizeInvoiceData,
            RequestOutAuthorizePaymentDetails,
            RequestOutAuthorizeVendorData,
        )

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.money_out.authorize_out(
                entry_point="48acde49",
                invoice_data=[
                    RequestOutAuthorizeInvoiceData(
                        bill_id=54323,
                    )
                ],
                order_description="Window Painting",
                payment_details=RequestOutAuthorizePaymentDetails(
                    total_amount=47.0,
                ),
                payment_method=AuthorizePaymentMethod(
                    method="managed",
                ),
                vendor_data=RequestOutAuthorizeVendorData(
                    vendor_number="7895433",
                ),
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.authorize_out(
            entry_point=entry_point,
            payment_method=payment_method,
            payment_details=payment_details,
            vendor_data=vendor_data,
            invoice_data=invoice_data,
            allow_duplicated_bills=allow_duplicated_bills,
            do_not_create_bills=do_not_create_bills,
            force_vendor_creation=force_vendor_creation,
            idempotency_key=idempotency_key,
            source=source,
            order_id=order_id,
            order_description=order_description,
            account_id=account_id,
            subdomain=subdomain,
            subscription_id=subscription_id,
            request_options=request_options,
        )
        return _response.data

    async def cancel_all_out(
        self, *, request: typing.Sequence[str], request_options: typing.Optional[RequestOptions] = None
    ) -> CaptureAllOutResponse:
        """
        Cancels an array of payout transactions.

        Parameters
        ----------
        request : typing.Sequence[str]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CaptureAllOutResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.money_out.cancel_all_out(
                request=["2-29", "2-28", "2-27"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel_all_out(request=request, request_options=request_options)
        return _response.data

    async def cancel_out(
        self, reference_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> PayabliApiResponse0000:
        """
        Cancel a payout transaction by ID.

        Parameters
        ----------
        reference_id : str
            The ID for the payout transaction.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        PayabliApiResponse0000
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.money_out.cancel_out(
                reference_id="129-219",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.cancel_out(reference_id, request_options=request_options)
        return _response.data

    async def capture_all_out(
        self,
        *,
        request: typing.Sequence[str],
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> CaptureAllOutResponse:
        """
        Captures an array of authorized payout transactions for settlement.

        Parameters
        ----------
        request : typing.Sequence[str]

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        CaptureAllOutResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.money_out.capture_all_out(
                request=["2-29", "2-28", "2-27"],
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.capture_all_out(
            request=request, idempotency_key=idempotency_key, request_options=request_options
        )
        return _response.data

    async def capture_out(
        self,
        reference_id: str,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AuthCapturePayoutResponse:
        """
        Captures a single authorized payout transaction by ID.

        Parameters
        ----------
        reference_id : str
            The ID for the payout transaction.

        idempotency_key : typing.Optional[IdempotencyKey]

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AuthCapturePayoutResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.money_out.capture_out(
                reference_id="129-219",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.capture_out(
            reference_id, idempotency_key=idempotency_key, request_options=request_options
        )
        return _response.data

    async def payout_details(
        self, trans_id: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> BillDetailResponse:
        """
        Returns details for a processed money out transaction.

        Parameters
        ----------
        trans_id : str
            ReferenceId for the transaction (PaymentId).

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        BillDetailResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.money_out.payout_details(
                trans_id="45-as456777hhhhhhhhhh77777777-324",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.payout_details(trans_id, request_options=request_options)
        return _response.data

    async def v_card_get(
        self, card_token: str, *, request_options: typing.Optional[RequestOptions] = None
    ) -> VCardGetResponse:
        """
        Retrieves vCard details for a single card in an entrypoint.

        Parameters
        ----------
        card_token : str
            ID for a virtual card.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        VCardGetResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.money_out.v_card_get(
                card_token="20230403315245421165",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.v_card_get(card_token, request_options=request_options)
        return _response.data

    async def send_v_card_link(
        self, *, trans_id: str, request_options: typing.Optional[RequestOptions] = None
    ) -> OperationResult:
        """
        Sends a virtual card link via email to the vendor associated with the `transId`.

        Parameters
        ----------
        trans_id : str
            The transaction ID of the virtual card payout. The ID is returned as `ReferenceId` in the response when you authorize a payout with POST /MoneyOut/authorize.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        OperationResult

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.money_out.send_v_card_link(
                trans_id="01K33Z6YQZ6GD5QVKZ856MJBSC",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.send_v_card_link(trans_id=trans_id, request_options=request_options)
        return _response.data

    async def get_check_image(self, asset_name: str, *, request_options: typing.Optional[RequestOptions] = None) -> str:
        """
        Retrieve the image of a check associated with a processed transaction.
        The check image is returned in the response body as a base64-encoded string.
        The check image is only available for payouts that have been processed.

        Parameters
        ----------
        asset_name : str
            Name of the check asset to retrieve. This is returned as `filename` in the `CheckData` object
            in the response when you make a GET request to `/MoneyOut/details/{transId}`.
            ```
                "CheckData": {
                  "ftype": "PDF",
                  "filename": "check133832686289732320_01JKBNZ5P32JPTZY8XXXX000000.pdf",
                  "furl": "",
                  "fContent": ""
              }
            ```

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            A base64-encoded string of the check image.

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.money_out.get_check_image(
                asset_name="check133832686289732320_01JKBNZ5P32JPTZY8XXXX000000.pdf",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_check_image(asset_name, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

from ..core.client_wrapper import AsyncClientWrapper, SyncClientWrapper
from ..core.request_options import RequestOptions
from ..types.attachments import Attachments
from ..types.email import Email
from ..types.idempotency_key import IdempotencyKey
from .raw_client import AsyncRawChargeBacksClient, RawChargeBacksClient
from .types.add_response_response import AddResponseResponse
from .types.chargeback_query_records import ChargebackQueryRecords

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ChargeBacksClient:
    def __init__(self, *, client_wrapper: SyncClientWrapper):
        self._raw_client = RawChargeBacksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> RawChargeBacksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        RawChargeBacksClient
        """
        return self._raw_client

    def add_response(
        self,
        id: int,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        attachments: typing.Optional[Attachments] = OMIT,
        contact_email: typing.Optional[Email] = OMIT,
        contact_name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddResponseResponse:
        """
        Add a response to a chargeback or ACH return.

        Parameters
        ----------
        id : int
            ID of the chargeback or return record.

        idempotency_key : typing.Optional[IdempotencyKey]

        attachments : typing.Optional[Attachments]
            Array of attached files to response.

        contact_email : typing.Optional[Email]
            Email of response submitter.

        contact_name : typing.Optional[str]
            Name of response submitter

        notes : typing.Optional[str]
            Response notes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddResponseResponse
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.charge_backs.add_response(
            id=1000000,
            idempotency_key="6B29FC40-CA47-1067-B31D-00DD010662DA",
        )
        """
        _response = self._raw_client.add_response(
            id,
            idempotency_key=idempotency_key,
            attachments=attachments,
            contact_email=contact_email,
            contact_name=contact_name,
            notes=notes,
            request_options=request_options,
        )
        return _response.data

    def get_chargeback(
        self, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ChargebackQueryRecords:
        """
        Retrieves a chargeback record and its details.

        Parameters
        ----------
        id : int
            ID of the chargeback or return record. This is returned as `chargebackId` in the [RecievedChargeback](/developers/developer-guides/webhook-payloads#receivedChargeback) and [ReceivedAchReturn](/developers/developer-guides/webhook-payloads#receivedachreturn) webhook notifications.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargebackQueryRecords
            Success

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.charge_backs.get_chargeback(
            id=1000000,
        )
        """
        _response = self._raw_client.get_chargeback(id, request_options=request_options)
        return _response.data

    def get_chargeback_attachment(
        self, file_name: str, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Parameters
        ----------
        file_name : str
            The chargeback attachment's file name.

        id : int
            The ID of chargeback or return record.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            OK

        Examples
        --------
        from payabli import payabli

        client = payabli(
            api_key="YOUR_API_KEY",
        )
        client.charge_backs.get_chargeback_attachment(
            file_name="fileName",
            id=1000000,
        )
        """
        _response = self._raw_client.get_chargeback_attachment(file_name, id, request_options=request_options)
        return _response.data


class AsyncChargeBacksClient:
    def __init__(self, *, client_wrapper: AsyncClientWrapper):
        self._raw_client = AsyncRawChargeBacksClient(client_wrapper=client_wrapper)

    @property
    def with_raw_response(self) -> AsyncRawChargeBacksClient:
        """
        Retrieves a raw implementation of this client that returns raw responses.

        Returns
        -------
        AsyncRawChargeBacksClient
        """
        return self._raw_client

    async def add_response(
        self,
        id: int,
        *,
        idempotency_key: typing.Optional[IdempotencyKey] = None,
        attachments: typing.Optional[Attachments] = OMIT,
        contact_email: typing.Optional[Email] = OMIT,
        contact_name: typing.Optional[str] = OMIT,
        notes: typing.Optional[str] = OMIT,
        request_options: typing.Optional[RequestOptions] = None,
    ) -> AddResponseResponse:
        """
        Add a response to a chargeback or ACH return.

        Parameters
        ----------
        id : int
            ID of the chargeback or return record.

        idempotency_key : typing.Optional[IdempotencyKey]

        attachments : typing.Optional[Attachments]
            Array of attached files to response.

        contact_email : typing.Optional[Email]
            Email of response submitter.

        contact_name : typing.Optional[str]
            Name of response submitter

        notes : typing.Optional[str]
            Response notes

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        AddResponseResponse
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.charge_backs.add_response(
                id=1000000,
                idempotency_key="6B29FC40-CA47-1067-B31D-00DD010662DA",
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.add_response(
            id,
            idempotency_key=idempotency_key,
            attachments=attachments,
            contact_email=contact_email,
            contact_name=contact_name,
            notes=notes,
            request_options=request_options,
        )
        return _response.data

    async def get_chargeback(
        self, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> ChargebackQueryRecords:
        """
        Retrieves a chargeback record and its details.

        Parameters
        ----------
        id : int
            ID of the chargeback or return record. This is returned as `chargebackId` in the [RecievedChargeback](/developers/developer-guides/webhook-payloads#receivedChargeback) and [ReceivedAchReturn](/developers/developer-guides/webhook-payloads#receivedachreturn) webhook notifications.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        ChargebackQueryRecords
            Success

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.charge_backs.get_chargeback(
                id=1000000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_chargeback(id, request_options=request_options)
        return _response.data

    async def get_chargeback_attachment(
        self, file_name: str, id: int, *, request_options: typing.Optional[RequestOptions] = None
    ) -> str:
        """
        Parameters
        ----------
        file_name : str
            The chargeback attachment's file name.

        id : int
            The ID of chargeback or return record.

        request_options : typing.Optional[RequestOptions]
            Request-specific configuration.

        Returns
        -------
        str
            OK

        Examples
        --------
        import asyncio

        from payabli import Asyncpayabli

        client = Asyncpayabli(
            api_key="YOUR_API_KEY",
        )


        async def main() -> None:
            await client.charge_backs.get_chargeback_attachment(
                file_name="fileName",
                id=1000000,
            )


        asyncio.run(main())
        """
        _response = await self._raw_client.get_chargeback_attachment(file_name, id, request_options=request_options)
        return _response.data



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class VendorResponseSummary(UniversalBaseModel):
    """
    Vendor bill summary statistics
    """

    active_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ActiveBills")] = None
    pending_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="PendingBills")] = None
    in_transit_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="InTransitBills")] = None
    paid_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="PaidBills")] = None
    overdue_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="OverdueBills")] = None
    approved_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="ApprovedBills")] = None
    disapproved_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="DisapprovedBills")] = None
    total_bills: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="TotalBills")] = None
    active_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="ActiveBillsAmount")
    ] = None
    pending_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="PendingBillsAmount")
    ] = None
    in_transit_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="InTransitBillsAmount")
    ] = None
    paid_bills_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="PaidBillsAmount")] = (
        None
    )
    overdue_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="OverdueBillsAmount")
    ] = None
    approved_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="ApprovedBillsAmount")
    ] = None
    disapproved_bills_amount: typing_extensions.Annotated[
        typing.Optional[float], FieldMetadata(alias="DisapprovedBillsAmount")
    ] = None
    total_bills_amount: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="TotalBillsAmount")] = (
        None
    )

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class PaypointSummary(UniversalBaseModel):
    amount_subs: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="amountSubs")] = None
    amount_tx: typing_extensions.Annotated[typing.Optional[float], FieldMetadata(alias="amountTx")] = None
    count_subs: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="countSubs")] = None
    count_tx: typing_extensions.Annotated[typing.Optional[int], FieldMetadata(alias="countTx")] = None
    customers: typing.Optional[int] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata


class AchSetup(UniversalBaseModel):
    accept_ccd: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="acceptCCD")] = pydantic.Field(
        default=None
    )
    """
    CCD is an ACH SEC Code that can be used in ACH transactions by the user that indicates the transaction is a Corporate Credit or Debit Entry. Options are: `true` and `false`
    """

    accept_ppd: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="acceptPPD")] = pydantic.Field(
        default=None
    )
    """
    PPD is an ACH SEC Code that can be used in ACH transactions by the user that indicates the transaction is a Prearranged Payment and Deposit. 
    """

    accept_web: typing_extensions.Annotated[typing.Optional[bool], FieldMetadata(alias="acceptWeb")] = pydantic.Field(
        default=None
    )
    """
    Web is an ACH SEC Code that can be used in ACH transactions by the user that indicates the transaction is a Internet Initiated/Mobile Entry Options are `true` and `false`.
    """

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow



# This file was auto-generated by Fern from our API Definition.

import typing

import pydantic
import typing_extensions
from ..core.pydantic_utilities import IS_PYDANTIC_V2, UniversalBaseModel
from ..core.serialization import FieldMetadata
from .template_element import TemplateElement
from .visible import Visible


class CardFeeSection(UniversalBaseModel):
    ach_batch_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="achBatchCardFee")
    ] = None
    annual_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="annualCardFee")
    ] = None
    avs_card_fee: typing_extensions.Annotated[typing.Optional[TemplateElement], FieldMetadata(alias="avsCardFee")] = (
        None
    )
    chargeback_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="chargebackCardFee")
    ] = None
    dda_rejects_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="ddaRejectsCardFee")
    ] = None
    early_termination_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="earlyTerminationCardFee")
    ] = None
    minimum_processing_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="minimumProcessingCardFee")
    ] = None
    monthly_pci_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="monthlyPCICardFee")
    ] = None
    montly_platform_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="montlyPlatformCardFee")
    ] = None
    retrieval_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="retrievalCardFee")
    ] = None
    transaction_card_fee: typing_extensions.Annotated[
        typing.Optional[TemplateElement], FieldMetadata(alias="transactionCardFee")
    ] = None
    visible: typing.Optional[Visible] = None

    if IS_PYDANTIC_V2:
        model_config: typing.ClassVar[pydantic.ConfigDict] = pydantic.ConfigDict(extra="allow", frozen=True)  # type: ignore # Pydantic v2
    else:

        class Config:
            frozen = True
            smart_union = True
            extra = pydantic.Extra.allow
